@@ -63,6 +63,9 @@ class CdoTest(unittest2.TestCase):
             cdo.setCdo(newCDO)
             self.assertEqual(newCDO,cdo.getCdo())
             cdo.setCdo('cdo')
+            # now constructor option
+            cdo = Cdo(cdo=newCDO)
+            self.assertEqual(newCDO,cdo.getCdo())
 
     def testDbg(self):
         if not 'DEBUG' in os.environ:
@@ -180,6 +183,20 @@ class CdoTest(unittest2.TestCase):
         ofile = cdo.setname("veloc", input=" -copy -random,r1x1",options = "-f nc")
         self.assertEqual(["veloc"],cdo.showname(input = ofile))
 
+    def test_pychain(self):
+        cdo = Cdo()
+        ofile = cdo.setname("veloc").copy.random("r1x1").add_option("-f nc").run()
+        self.assertEqual(["veloc"],cdo.showname(input = ofile))
+
+    def test_pychain2(self):
+        # compare the two different ways
+        cdo = Cdo()
+        ofile1 = cdo.setname("veloc").copy.random("r1x1").add_option("-f nc").run()
+        cdo = Cdo()
+        ofile2 = cdo.setname("veloc", input=" -copy -random,r1x1",options = "-f nc")
+        diff = cdo.diff(input=[ofile1, ofile2])
+        self.assertFalse(diff, msg=diff)
+
     def test_diff(self):
         cdo = Cdo()
         cdo.debug = DEBUG
@@ -260,15 +277,6 @@ class CdoTest(unittest2.TestCase):
        #sum = cdo.fldsum(input = cdo.stdatm("0",options="-f nc"),returnCdf=True)
        #self.assertEqual(288.0,sum.variables["T"][:])
 
-    def test_cdf(self):
-        cdo = Cdo()
-        self.assertTrue(hasattr(cdo, "cdf"))# not in cdo.__dict__)
-        if cdo.hasNetcdf:
-          sum = cdo.fldsum(input = cdo.stdatm("0",options="-f nc"),returnCdf=True)
-          self.assertEqual(1013.25,sum.variables["P"][:])
-        else:
-          self.assertRaises(ImportError,cdo.fldsum,input = cdo.stdatm("0",options="-f nc"),returnCdf=True)
-
     def test_thickness(self):
         cdo = Cdo()
         levels            = "25 100 250 500 875 1400 2100 3000 4000 5000".split(' ')
@@ -443,7 +451,7 @@ class CdoTest(unittest2.TestCase):
         cdo.__print__('test_errorException')
         self.assertFalse(hasattr(cdo, 'nonExistingMethod'))
         self.assertFalse(not 'max' in cdo.operators)
-        self.failUnlessRaises(CDOException, cdo.max)
+        self.failUnlessRaises(CDOException, cdo.max.read)
         try:
             cdo.max()
         except CDOException as e:
@@ -603,10 +611,11 @@ class CdoTest(unittest2.TestCase):
           print(cdo)
         if cdo.hasNetcdf:
           bathy = cdo.setrtomiss(0,10000,
-                                 input = cdo.topo('r100x100'),returnMaArray='var1')
+                                 input = cdo.topo('r100x100').run(),
+                                 returnMaArray='var1')
           plot(bathy)
           oro = cdo.setrtomiss(-10000,0,
-                               input = cdo.topo(),returnMaArray='var1')
+                               input = cdo.topo().run(),returnMaArray='var1')
           plot(oro)
           random = cdo.setname('test_maArray',
                                input = "-setrtomiss,0.4,0.8 -random,r180x90 ",
@@ -796,8 +805,11 @@ class CdoTest(unittest2.TestCase):
 
       def test_longChain(self):
         cdo = Cdo()
+        dataCreationOperator = 'seq'
+        if (parse_version(cdo.version()) < parse_version('1.9.8')):
+          dataCreationOperator = 'for'
         if cdo.hasNetcdf:
-          ifile = "-enlarge,global_0.3 -settaxis,2000-01-01 -expr,'t=sin(for*3.141529/180.0)' -for,1,10"
+          ifile = "-enlarge,global_0.3 -settaxis,2000-01-01 -expr,'t=sin({0}*3.141529/180.0)' -{0},1,10".format(dataCreationOperator)
           t = cdo.fldmax(input="-div -sub -timmean -seltimestep,2,3 %s -seltimestep,1 %s -gridarea %s"%(ifile,ifile,ifile),
               returnMaArray="t")
           self.assertTrue(abs(8.9813e-09 - t[0][0][0]) < 1.0e-10, 'Found non-zero diff')
@@ -828,7 +840,10 @@ class CdoTest(unittest2.TestCase):
         cdfFile = cdo.copy(options="-f nc",input=input)
         if cdo.hasNetcdf:
           cdf     = cdo.readCdf(cdfFile)
-          self.assertEqual(sorted(['lat','lon','for','time']),sorted(list(cdf.variables.keys())))
+          if (parse_version(cdo.version()) < parse_version('1.9.8')):
+            self.assertEqual(sorted(['lat','lon','for','time']),sorted(list(cdf.variables.keys())))
+          else:
+            self.assertEqual(sorted(['lat','lon','seq','time']),sorted(list(cdf.variables.keys())))
 
       def test_phc(self):
         ifile = "-select,level=0 " + DATA_DIR + '/icon/phc.nc'
@@ -886,6 +901,19 @@ class CdoTest(unittest2.TestCase):
 
           smooth20 = cdo.smooth('nsmooth=20',input="-sellonlatbox,0,30,0,90 -chname,SO,s,TempO,t " + ifile, returnMaArray='s',options='-f nc')
           plot(np.flipud(smooth20[0,:,:]),ofile='smooth20',title='smooth,nsmooth=20')
+
+      def test_ydiv(self):
+        cdo = Cdo()
+        cdo.debug = True
+        if ('yeardiv' in cdo.operators):
+          input = "-expr,'seq=seq*cyear(seq)/seq;' -settaxis,2001-01-01,12:00:00,12hours -for,1,10000 -yearmean -expr,'seq=seq*cyear(seq)/seq;' -settaxis,2001-01-01,12:00:00,12hours -for,1,10000"
+          values = cdo.yeardiv(input=input,returnArray='seq')
+          print(np.zeros(len(values))+1.0)
+          print(np.ravel(values))
+          self.assertTrue((np.zeros(len(values))+1.0 == np.ravel(values)).all())
+        else:
+          print("no tests for 'yeardiv': operator does not exist")
+
 #===============================================================================
 if __name__ == '__main__':
     suite = unittest2.TestLoader().loadTestsFromTestCase(CdoTest)
